'' +--------------------------------------------------------------------------+
'' | Cluso's Demo ILI9163 1.44in SPI 128*128 V1.1 LCD Driver           V0.60+ |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2015-2016  "Cluso99" (Ray Rodrick)                    |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' RR20150101       Commence
''                  Screen is 128*128 offset by 32
'' RR20150121  023  drawLine, drawCircle, drawPixel, setWindow, fillWindow, fillRectangle, lcd.DrawChar working
''             030  clock working (fast, no time set, etc)  
''             032  digital clock
''             033  double height clock
''             035  tripple height double width clock
''             036  now use black pcb version (upside down and no offset)
''             038  simplify support routines
'' RR20150130  040  demo release
'' RR20150219  050  power LCD from prop pins (~2.0mA) [DID NOT WORK - must want 5V not 3V3 else change params!!!]
'' RR20161011  051  P8XBlade2 (power from prop pins - v050 didn't enable PWR/GND DIRA pins) 
'' RR20161011  052  create lcdSetup routine in low level routines (ready for pasm)
''             053  some fast pasm routines wkg in separate cog (delays added for viewing)
''             054  code tidy
'' RR20161015  055  Release
'' +--------------------------------------------------------------------------+
'' RR20161015  Split "ILI9163_spi_060.spin" v0.60 into separate files
''                "LCD_ILI9163_SPI_100.spin"  PASM Cog Driver &
''                "Demo_ILI9163_SPI_060.spin" SPIN Demo program
'' +--------------------------------------------------------------------------+
'' RR20161015  060a  Split into separate LCD PASM Cog Driver
''                   Remove Large Digital Clock (removed routine to display big numerals)
'' RR20161021        add pcb & rotation parameter to Start
''                   Round clock advance hour hand every 12 minutes
'' RR20161022  061   Release with "LCD_ILI9163_SPI_v100.spin"


''+-----------------------------------------------------+
''| Notes:                                              |
''+-----------------------------------------------------+
''  1. Supports both the RED and BLACK LCD boards
''      Set "LCD_pcb" & "LCD_rotate" in CON accordingly!!
''
  
CON

' _XINFREQ      = 5_000_000         '\ set for 5.00MHz (default)
' _CLKMODE      = xtal1 + pll16x    '/

' _XINFREQ      = 6_000_000         '\ set for 6.00MHz
' _CLKMODE      = xtal1 + pll16x    '/

' _XINFREQ      = 6_500_000         '\ set for 6.50MHz (CpuBlade/RamBlade/RamBlade3/TriBlade#2)
' _CLKMODE      = xtal1 + pll16x    '/

  _XINFREQ      = 12_000_000        '\ set for 12.00MHz (P8XBlade2)
  _CLKMODE      = xtal1 + pll8x     '/

  LCD_rotate_0          = $00       '\   0 degrees    (red is rotated 180deg to black)
  LCD_rotate_90         = $A0       '|  90 degrees
  LCD_rotate_180        = $C0       '| 180 degrees
  LCD_rotate_270        = $60       '/ 270 degrees
  LCD_blackpcb          = 0         '\
  LCD_redpcb            = 32        '/ 

' Set the following to suit...
  LCD_rotate    = LCD_rotate_0
  LCD_pcb       = LCD_blackpcb


' ILI9163C registers...
  LCD_NOP               = $00
  LCD_SW_RESET          = $01   
  LCD_SLEEP_ON          = $10
  LCD_SLEEP_OFF         = $11
  LCD_PARTIAL           = $12
  LCD_NORMAL            = $13
  LCD_INVERSION_OFF     = $20
  LCD_INVERSION_ON      = $21
  LCD_GAMMA_SET         = $26
  LCD_DISPLAY_OFF       = $28
  LCD_DISPLAY_ON        = $29
  LCD_COL_ADR           = $2A
  LCD_ROW_ADR           = $2B
  LCD_RAM_WRITE         = $2C
  LCD_COLOR_SPACE       = $2D   '4K/65K/262K
' LCD_RAMRD             = $2E
  LCD_MADCTR            = $36   'Mem Addr Contol (rotate screen)
  LCD_PIXEL_FORMAT      = $3A
  LCD_FRAME_CTL1        = $B1
  LCD_INVERSION_CTL     = $B4
  LCD_POWER_CTL1        = $C0
  LCD_POWER_CTL2        = $C1
  LCD_VCOM_CTL1         = $C5
  LCD_VCOM_OFFSET       = $C7
  LCD_POS_GAMMA         = $E0
  LCD_NEG_GAMMA         = $E1
  LCD_GAMMA_RS          = $F2
' LCD_ALL_ON            = $A4
' LCD_ALL_OFF           = $A5



' BGR 5+6+5 16bits/pixel (RGB reversed)
  black = $0000
  white = $FFFF
  blue  = $F800
  green = $07E0
  red   = $001F
  yellow= red+green
  purple= red+blue

'                 ' sin(x) * 1024    ' degrees
Z00     =    0    ' 0.0000 * 1024    '  0 
Z01     =  107    ' 0.1045 * 1024    '  6
Z02     =  213    ' 0.2079 * 1024    ' 12
Z03     =  316    ' 0.3090 * 1024    ' 18
Z04     =  416    ' 0.4067 * 1024    ' 24
Z05     =  512    ' 0.5000 * 1024    ' 30
Z06     =  602    ' 0.5878 * 1024    ' 36
Z07     =  685    ' 0.6691 * 1024    ' 42
Z08     =  761    ' 0.7431 * 1024    ' 48
Z09     =  828    ' 0.8090 * 1024    ' 54
Z10     =  887    ' 0.8660 * 1024    ' 60
Z11     =  935    ' 0.9135 * 1024    ' 66
Z12     =  974    ' 0.9511 * 1024    ' 72
Z13     = 1002    ' 0.9781 * 1024    ' 78
Z14     = 1018    ' 0.9945 * 1024    ' 84
Z15     = 1024    ' 1.0000 * 1024    ' 90
'180    =     ' 0.0000 * 1024    '    180
'270    =     '-1.0000 * 1024    '    270

' Clock constants
_CX     = 64                    '\ clock center
_CY     = 64                    '/
_CD     = 120                   ' clock  face dia
_CS     = 45                    ' second hand length
_CM     = 40                    ' minute hand length
_CH     = 30                    ' hour   hand length
fgd     = green                 ' foreground dial color (clock dial)
bgd     = black                 ' background dial color
fgh     = red                   ' hour hand color
fgm     = yellow                ' minute hand color
fgs     = white                 ' second hand color


OBJ
  lcd:  "LCD_ILI9163_SPI_v100"          ' LCD pasm cog driver

VAR
  long  cog                                             ' pasm cog+1

PUB start 

  cog := lcd.Start(LCD_pcb, LCD_rotate)                 ' sw/hw initialise LCD (0/32=black/red pcb, rotate 0/90/180/270)

' screen defaults...
' lcd.setFGcolor(blue)                                  ' set foreground color
' lcd.setBGcolor(black)                                 ' set background color
' lcd.clearScreen                                       ' clear screen (background color)



'Select one/some of the following...
  waitcnt(cnt+clkfreq)                                  ' wait
  drawSomeChars
  waitcnt(cnt+clkfreq)                                  ' wait
  drawSomeLines
  waitcnt(cnt+clkfreq)                                  ' wait
  drawSomeBoxes
  waitcnt(cnt+clkfreq)                                  ' wait
  drawSomeCircles
  waitcnt(cnt+clkfreq)                                  ' wait

  main_roundclock


PRI drawSomeChars | i
' Draw some chars

  lcd.clearScreen
  lcd.setFGcolor(white)
  lcd.setBGcolor(green)
  
  repeat i from 0 to 255
    lcd.DrawChar(" "+i)


PRI drawSomeLines
' Draw some lines

  lcd.setBGcolor(black)
  lcd.clearScreen
  
  drawLine(0, 0, 127, 127, blue)
  drawLine(0, 127, 127, 0, blue)  
  drawLine(20, 20, 107, 107, red)
  drawLine(20, 107, 107, 20, red)
  drawLine(20, 63, 107, 63, yellow)
  drawLine(63, 20, 63, 107, yellow)


PRI drawSomeBoxes | xs, ys, xe, ye
' Draw some diagonal boxes

  lcd.clearScreen
  
  lcd.SetWindow($00,$00,$1F,$1F)
  lcd.FillWindow(white)

  lcd.SetWindow($10,$10,$2F,$2F)
  lcd.FillWindow(red+green)     ' yellow

  lcd.SetWindow($20,$20,$3F,$3F)
  lcd.FillWindow(blue)

  lcd.SetWindow($30,$30,$4F,$4F)
  lcd.FillWindow(green)

  lcd.SetWindow($40,$40,$5F,$5F)
  lcd.FillWindow(red)

  lcd.SetWindow($50,$50,$6F,$6F)
  lcd.FillWindow(red+blue)      ' purple

  lcd.SetWindow($60,$60,$7F,$7F)
  lcd.FillWindow(blue+green)    ' light blue

  lcd.SetWindow($70,$70,$8F,$8F)                        ' part outside screen
  lcd.FillWindow(red+(blue/2))  ' pink

  xs := 0
  ys := 128-16
  xe := 16
  ye := 128-1
  repeat 8
    lcd.SetWindow(xs,ys,xe,ye)
    lcd.FillWindow(red+green)
    xs += 16
    ys -= 16
    xe += 16
    ye -= 16


PRI drawSomeCircles
' Draw some circles

  lcd.clearScreen
  
  drawCircle(64, 64, 60, green)
  drawCircle(64, 64, 50, red)
  drawCircle(64, 64, 40, blue)

  
''+-----------------------------------------------------+
''| High Level Drivers                                  |
''+-----------------------------------------------------+
  
PRI drawLine(xs, ys, xe, ye, rgb) | i, x, y
' Draw Line - start co-ords, end co-ords, color    
  'plot incrementing x axis
  repeat i from 0 to xe-xs
    y := (ye-ys)*i/(xe-xs)
    drawPixel(xs+i, ys+y, rgb)
  'plot incrementing y axis  
  repeat i from 0 to ye-ys
    x := (xe-xs)*i/(ye-ys)
    drawPixel(xs+x, ys+i, rgb)    

PRI drawCircle(xc, yc, dia, rgb) | i, x, y, d
' Draw Circle d^2 = x^2 + y^2; y = ^^(d^2 - x^2) (Pythagoras theorum)
  repeat i from 0 to dia-1
    x := i
    y := ^^((dia*dia) - (x*x))
    'plot incrementing x axis
    drawPixel(x+xc, y+yc, rgb)
    drawPixel(xc-x, y+yc, rgb)
    drawPixel(x+xc, yc-y, rgb)
    drawPixel(xc-x, yc-y, rgb)
    'plot incrementing y axis
    drawPixel(y+yc, x+xc, rgb)
    drawPixel(yc-y, x+xc, rgb)
    drawPixel(y+yc, yc-x, rgb)
    drawPixel(yc-y, yc-x, rgb)

PRI drawPixel(x, y, rgb) | i
' Draw 1 pixel
  lcd.setWindow(x, y, x, y)                             ' 1 pixel
  lcd.fillWindow(rgb)

PRI fillRectangle(xs, ys, xe, ye, rgb)
  lcd.setWindow(xs, ys, xe, ye)
  lcd.fillWindow(rgb)


''+-----------------------------------------------------+
''| Special Functions...                                |
''+-----------------------------------------------------+
  
PRI main_roundclock | h, hprev, m, mprev, s, sprev, tick, hx, hz
' Round Clock

  lcd.setFGcolor(fgd)
  lcd.setBGcolor(bgd)
  lcd.clearScreen

' Draw Clock Face
  drawCircle(_CX, _CY, _CD/2, fgd)
  lcd.setColRow( 56,   8)
  lcd.DrawChar("1")
  lcd.DrawChar("2")
  lcd.setColRow( 31,  16)
  lcd.DrawChar("1")
  lcd.DrawChar("1")
  lcd.setColRow( 89,  16)
  lcd.DrawChar("1")
  lcd.setColRow( 15,  36)
  lcd.DrawChar("1")
  lcd.DrawChar("0")
  lcd.setColRow(106,  36)
  lcd.DrawChar("2")
  lcd.setColRow(  8,  60)
  lcd.DrawChar("9")
  lcd.setColRow(112,  60)
  lcd.DrawChar("3")
  lcd.setColRow( 15,  84)
  lcd.DrawChar("8")
  lcd.setColRow(106,  84)
  lcd.DrawChar("4")
  lcd.setColRow( 31, 104)
  lcd.DrawChar("7")
  lcd.setColRow( 89, 104)
  lcd.DrawChar("5")
  lcd.setColRow( 60, 112)
  lcd.DrawChar("6")

  tick := cnt

' Draw Clock Hands
  hprev~
  mprev~
  sprev~
  repeat h from 0 to 11
    hx := h*5                                                                   ' hx= 0..59 for hour hand angle
    drawLine(_CX, _CY, byte[@HH00][hprev*2],  byte[@HH00][hprev*2+1],  bgd)     ' remove prev hour hand
    drawLine(_CX, _CY, byte[@HH00][hx*2],     byte[@HH00][hx*2+1],     bgd)     ' show hour hand
    hprev := hx
    repeat m from 0 to 59
      drawLine(_CX, _CY, byte[@MH00][mprev*2], byte[@MH00][mprev*2+1], bgd)     ' remove prev minute hand
      drawLine(_CX, _CY, byte[@MH00][m*2],     byte[@MH00][m*2+1],     fgm)     ' show minute hand
      case m                                                                    
        12,24,36,48:                                                            '\ hx is +0/1/2/3/4 offset...
          hx++                                                                  '| ...for hour hand to...
          drawLine(_CX, _CY, byte[@HH00][hprev*2], byte[@HH00][hprev*2+1], bgd) '| ...advance every...
          drawLine(_CX, _CY, byte[@HH00][hx*2], byte[@HH00][hx*2+1], fgh)       '| ...12 minutes...
          hprev := hx                                                           '/ ...
      mprev := m
      repeat s from 0 to 59
        drawLine(_CX, _CY, byte[@SH00][sprev*2], byte[@SH00][sprev*2+1], bgd)   ' remove prev second hand
        drawLine(_CX, _CY, byte[@SH00][s*2],     byte[@SH00][s*2+1],     fgs)   ' show second hand
        drawLine(_CX, _CY, byte[@MH00][m*2],     byte[@MH00][m*2+1],     fgm)   ' show minute hand
        drawLine(_CX, _CY, byte[@HH00][hx*2],    byte[@HH00][hx*2+1],    fgh)   ' show hour hand
        sprev := s
        waitcnt(tick += clkfreq)                                                ' +1s


DAT
        org             0
' Hands (seconds) x & y co-ords
SH00    byte  _CX + ((_CS * Z00) >> 10), _CY - ((_CS * Z15) >> 10) '00
        byte  _CX + ((_CS * Z01) >> 10), _CY - ((_CS * Z14) >> 10) '01
        byte  _CX + ((_CS * Z02) >> 10), _CY - ((_CS * Z13) >> 10) '02
        byte  _CX + ((_CS * Z03) >> 10), _CY - ((_CS * Z12) >> 10) '...
        byte  _CX + ((_CS * Z04) >> 10), _CY - ((_CS * Z11) >> 10)
        byte  _CX + ((_CS * Z05) >> 10), _CY - ((_CS * Z10) >> 10)
        byte  _CX + ((_CS * Z06) >> 10), _CY - ((_CS * Z09) >> 10)
        byte  _CX + ((_CS * Z07) >> 10), _CY - ((_CS * Z08) >> 10)
        byte  _CX + ((_CS * Z08) >> 10), _CY - ((_CS * Z07) >> 10)
        byte  _CX + ((_CS * Z09) >> 10), _CY - ((_CS * Z06) >> 10)
        byte  _CX + ((_CS * Z10) >> 10), _CY - ((_CS * Z05) >> 10)
        byte  _CX + ((_CS * Z11) >> 10), _CY - ((_CS * Z04) >> 10)
        byte  _CX + ((_CS * Z12) >> 10), _CY - ((_CS * Z03) >> 10)
        byte  _CX + ((_CS * Z13) >> 10), _CY - ((_CS * Z02) >> 10)
        byte  _CX + ((_CS * Z14) >> 10), _CY - ((_CS * Z01) >> 10)
                                                              
        byte  _CX + ((_CS * Z15) >> 10), _CY + ((_CS * Z00) >> 10)
        byte  _CX + ((_CS * Z14) >> 10), _CY + ((_CS * Z01) >> 10)
        byte  _CX + ((_CS * Z13) >> 10), _CY + ((_CS * Z02) >> 10)
        byte  _CX + ((_CS * Z12) >> 10), _CY + ((_CS * Z03) >> 10)
        byte  _CX + ((_CS * Z11) >> 10), _CY + ((_CS * Z04) >> 10)
        byte  _CX + ((_CS * Z10) >> 10), _CY + ((_CS * Z05) >> 10)
        byte  _CX + ((_CS * Z09) >> 10), _CY + ((_CS * Z06) >> 10)
        byte  _CX + ((_CS * Z08) >> 10), _CY + ((_CS * Z07) >> 10)
        byte  _CX + ((_CS * Z07) >> 10), _CY + ((_CS * Z08) >> 10)
        byte  _CX + ((_CS * Z06) >> 10), _CY + ((_CS * Z09) >> 10)
        byte  _CX + ((_CS * Z05) >> 10), _CY + ((_CS * Z10) >> 10)
        byte  _CX + ((_CS * Z04) >> 10), _CY + ((_CS * Z11) >> 10)
        byte  _CX + ((_CS * Z03) >> 10), _CY + ((_CS * Z12) >> 10)
        byte  _CX + ((_CS * Z02) >> 10), _CY + ((_CS * Z13) >> 10)
        byte  _CX + ((_CS * Z01) >> 10), _CY + ((_CS * Z14) >> 10)
    
        byte  _CX - ((_CS * Z00) >> 10), _CY + ((_CS * Z15) >> 10)
        byte  _CX - ((_CS * Z01) >> 10), _CY + ((_CS * Z14) >> 10)
        byte  _CX - ((_CS * Z02) >> 10), _CY + ((_CS * Z13) >> 10)
        byte  _CX - ((_CS * Z03) >> 10), _CY + ((_CS * Z12) >> 10)
        byte  _CX - ((_CS * Z04) >> 10), _CY + ((_CS * Z11) >> 10)
        byte  _CX - ((_CS * Z05) >> 10), _CY + ((_CS * Z10) >> 10)
        byte  _CX - ((_CS * Z06) >> 10), _CY + ((_CS * Z09) >> 10)
        byte  _CX - ((_CS * Z07) >> 10), _CY + ((_CS * Z08) >> 10)
        byte  _CX - ((_CS * Z08) >> 10), _CY + ((_CS * Z07) >> 10)
        byte  _CX - ((_CS * Z09) >> 10), _CY + ((_CS * Z06) >> 10)
        byte  _CX - ((_CS * Z10) >> 10), _CY + ((_CS * Z05) >> 10)
        byte  _CX - ((_CS * Z11) >> 10), _CY + ((_CS * Z04) >> 10)
        byte  _CX - ((_CS * Z12) >> 10), _CY + ((_CS * Z03) >> 10)
        byte  _CX - ((_CS * Z13) >> 10), _CY + ((_CS * Z02) >> 10)
        byte  _CX - ((_CS * Z14) >> 10), _CY + ((_CS * Z01) >> 10)
                                                                  
        byte  _CX - ((_CS * Z15) >> 10), _CY - ((_CS * Z00) >> 10)
        byte  _CX - ((_CS * Z14) >> 10), _CY - ((_CS * Z01) >> 10)
        byte  _CX - ((_CS * Z13) >> 10), _CY - ((_CS * Z02) >> 10)
        byte  _CX - ((_CS * Z12) >> 10), _CY - ((_CS * Z03) >> 10)
        byte  _CX - ((_CS * Z11) >> 10), _CY - ((_CS * Z04) >> 10)
        byte  _CX - ((_CS * Z10) >> 10), _CY - ((_CS * Z05) >> 10)
        byte  _CX - ((_CS * Z09) >> 10), _CY - ((_CS * Z06) >> 10)
        byte  _CX - ((_CS * Z08) >> 10), _CY - ((_CS * Z07) >> 10)
        byte  _CX - ((_CS * Z07) >> 10), _CY - ((_CS * Z08) >> 10)
        byte  _CX - ((_CS * Z06) >> 10), _CY - ((_CS * Z09) >> 10)
        byte  _CX - ((_CS * Z05) >> 10), _CY - ((_CS * Z10) >> 10)
        byte  _CX - ((_CS * Z04) >> 10), _CY - ((_CS * Z11) >> 10)
        byte  _CX - ((_CS * Z03) >> 10), _CY - ((_CS * Z12) >> 10)
        byte  _CX - ((_CS * Z02) >> 10), _CY - ((_CS * Z13) >> 10)
        byte  _CX - ((_CS * Z01) >> 10), _CY - ((_CS * Z14) >> 10)

' Hands (minutes) x & y co-ords
MH00    byte  _CX + ((_CM * Z00) >> 10), _CY - ((_CM * Z15) >> 10) '00
        byte  _CX + ((_CM * Z01) >> 10), _CY - ((_CM * Z14) >> 10) '01
        byte  _CX + ((_CM * Z02) >> 10), _CY - ((_CM * Z13) >> 10) '02
        byte  _CX + ((_CM * Z03) >> 10), _CY - ((_CM * Z12) >> 10) '...
        byte  _CX + ((_CM * Z04) >> 10), _CY - ((_CM * Z11) >> 10)
        byte  _CX + ((_CM * Z05) >> 10), _CY - ((_CM * Z10) >> 10)
        byte  _CX + ((_CM * Z06) >> 10), _CY - ((_CM * Z09) >> 10)
        byte  _CX + ((_CM * Z07) >> 10), _CY - ((_CM * Z08) >> 10)
        byte  _CX + ((_CM * Z08) >> 10), _CY - ((_CM * Z07) >> 10)
        byte  _CX + ((_CM * Z09) >> 10), _CY - ((_CM * Z06) >> 10)
        byte  _CX + ((_CM * Z10) >> 10), _CY - ((_CM * Z05) >> 10)
        byte  _CX + ((_CM * Z11) >> 10), _CY - ((_CM * Z04) >> 10)
        byte  _CX + ((_CM * Z12) >> 10), _CY - ((_CM * Z03) >> 10)
        byte  _CX + ((_CM * Z13) >> 10), _CY - ((_CM * Z02) >> 10)
        byte  _CX + ((_CM * Z14) >> 10), _CY - ((_CM * Z01) >> 10)
                                                              
        byte  _CX + ((_CM * Z15) >> 10), _CY + ((_CM * Z00) >> 10)
        byte  _CX + ((_CM * Z14) >> 10), _CY + ((_CM * Z01) >> 10)
        byte  _CX + ((_CM * Z13) >> 10), _CY + ((_CM * Z02) >> 10)
        byte  _CX + ((_CM * Z12) >> 10), _CY + ((_CM * Z03) >> 10)
        byte  _CX + ((_CM * Z11) >> 10), _CY + ((_CM * Z04) >> 10)
        byte  _CX + ((_CM * Z10) >> 10), _CY + ((_CM * Z05) >> 10)
        byte  _CX + ((_CM * Z09) >> 10), _CY + ((_CM * Z06) >> 10)
        byte  _CX + ((_CM * Z08) >> 10), _CY + ((_CM * Z07) >> 10)
        byte  _CX + ((_CM * Z07) >> 10), _CY + ((_CM * Z08) >> 10)
        byte  _CX + ((_CM * Z06) >> 10), _CY + ((_CM * Z09) >> 10)
        byte  _CX + ((_CM * Z05) >> 10), _CY + ((_CM * Z10) >> 10)
        byte  _CX + ((_CM * Z04) >> 10), _CY + ((_CM * Z11) >> 10)
        byte  _CX + ((_CM * Z03) >> 10), _CY + ((_CM * Z12) >> 10)
        byte  _CX + ((_CM * Z02) >> 10), _CY + ((_CM * Z13) >> 10)
        byte  _CX + ((_CM * Z01) >> 10), _CY + ((_CM * Z14) >> 10)
    
        byte  _CX - ((_CM * Z00) >> 10), _CY + ((_CM * Z15) >> 10)
        byte  _CX - ((_CM * Z01) >> 10), _CY + ((_CM * Z14) >> 10)
        byte  _CX - ((_CM * Z02) >> 10), _CY + ((_CM * Z13) >> 10)
        byte  _CX - ((_CM * Z03) >> 10), _CY + ((_CM * Z12) >> 10)
        byte  _CX - ((_CM * Z04) >> 10), _CY + ((_CM * Z11) >> 10)
        byte  _CX - ((_CM * Z05) >> 10), _CY + ((_CM * Z10) >> 10)
        byte  _CX - ((_CM * Z06) >> 10), _CY + ((_CM * Z09) >> 10)
        byte  _CX - ((_CM * Z07) >> 10), _CY + ((_CM * Z08) >> 10)
        byte  _CX - ((_CM * Z08) >> 10), _CY + ((_CM * Z07) >> 10)
        byte  _CX - ((_CM * Z09) >> 10), _CY + ((_CM * Z06) >> 10)
        byte  _CX - ((_CM * Z10) >> 10), _CY + ((_CM * Z05) >> 10)
        byte  _CX - ((_CM * Z11) >> 10), _CY + ((_CM * Z04) >> 10)
        byte  _CX - ((_CM * Z12) >> 10), _CY + ((_CM * Z03) >> 10)
        byte  _CX - ((_CM * Z13) >> 10), _CY + ((_CM * Z02) >> 10)
        byte  _CX - ((_CM * Z14) >> 10), _CY + ((_CM * Z01) >> 10)
                                                                  
        byte  _CX - ((_CM * Z15) >> 10), _CY - ((_CM * Z00) >> 10)
        byte  _CX - ((_CM * Z14) >> 10), _CY - ((_CM * Z01) >> 10)
        byte  _CX - ((_CM * Z13) >> 10), _CY - ((_CM * Z02) >> 10)
        byte  _CX - ((_CM * Z12) >> 10), _CY - ((_CM * Z03) >> 10)
        byte  _CX - ((_CM * Z11) >> 10), _CY - ((_CM * Z04) >> 10)
        byte  _CX - ((_CM * Z10) >> 10), _CY - ((_CM * Z05) >> 10)
        byte  _CX - ((_CM * Z09) >> 10), _CY - ((_CM * Z06) >> 10)
        byte  _CX - ((_CM * Z08) >> 10), _CY - ((_CM * Z07) >> 10)
        byte  _CX - ((_CM * Z07) >> 10), _CY - ((_CM * Z08) >> 10)
        byte  _CX - ((_CM * Z06) >> 10), _CY - ((_CM * Z09) >> 10)
        byte  _CX - ((_CM * Z05) >> 10), _CY - ((_CM * Z10) >> 10)
        byte  _CX - ((_CM * Z04) >> 10), _CY - ((_CM * Z11) >> 10)
        byte  _CX - ((_CM * Z03) >> 10), _CY - ((_CM * Z12) >> 10)
        byte  _CX - ((_CM * Z02) >> 10), _CY - ((_CM * Z13) >> 10)
        byte  _CX - ((_CM * Z01) >> 10), _CY - ((_CM * Z14) >> 10)


' Hands (hours) x & y co-ords
HH00    byte  _CX + ((_CH * Z00) >> 10), _CY - ((_CH * Z15) >> 10) '00 
        byte  _CX + ((_CH * Z01) >> 10), _CY - ((_CH * Z14) >> 10) '01 
        byte  _CX + ((_CH * Z02) >> 10), _CY - ((_CH * Z13) >> 10) '02 
        byte  _CX + ((_CH * Z03) >> 10), _CY - ((_CH * Z12) >> 10) '...
        byte  _CX + ((_CH * Z04) >> 10), _CY - ((_CH * Z11) >> 10)
        byte  _CX + ((_CH * Z05) >> 10), _CY - ((_CH * Z10) >> 10)
        byte  _CX + ((_CH * Z06) >> 10), _CY - ((_CH * Z09) >> 10)
        byte  _CX + ((_CH * Z07) >> 10), _CY - ((_CH * Z08) >> 10)
        byte  _CX + ((_CH * Z08) >> 10), _CY - ((_CH * Z07) >> 10)
        byte  _CX + ((_CH * Z09) >> 10), _CY - ((_CH * Z06) >> 10)
        byte  _CX + ((_CH * Z10) >> 10), _CY - ((_CH * Z05) >> 10)
        byte  _CX + ((_CH * Z11) >> 10), _CY - ((_CH * Z04) >> 10)
        byte  _CX + ((_CH * Z12) >> 10), _CY - ((_CH * Z03) >> 10)
        byte  _CX + ((_CH * Z13) >> 10), _CY - ((_CH * Z02) >> 10)
        byte  _CX + ((_CH * Z14) >> 10), _CY - ((_CH * Z01) >> 10)
                                                              
        byte  _CX + ((_CH * Z15) >> 10), _CY + ((_CH * Z00) >> 10)
        byte  _CX + ((_CH * Z14) >> 10), _CY + ((_CH * Z01) >> 10)
        byte  _CX + ((_CH * Z13) >> 10), _CY + ((_CH * Z02) >> 10)
        byte  _CX + ((_CH * Z12) >> 10), _CY + ((_CH * Z03) >> 10)
        byte  _CX + ((_CH * Z11) >> 10), _CY + ((_CH * Z04) >> 10)
        byte  _CX + ((_CH * Z10) >> 10), _CY + ((_CH * Z05) >> 10)
        byte  _CX + ((_CH * Z09) >> 10), _CY + ((_CH * Z06) >> 10)
        byte  _CX + ((_CH * Z08) >> 10), _CY + ((_CH * Z07) >> 10)
        byte  _CX + ((_CH * Z07) >> 10), _CY + ((_CH * Z08) >> 10)
        byte  _CX + ((_CH * Z06) >> 10), _CY + ((_CH * Z09) >> 10)
        byte  _CX + ((_CH * Z05) >> 10), _CY + ((_CH * Z10) >> 10)
        byte  _CX + ((_CH * Z04) >> 10), _CY + ((_CH * Z11) >> 10)
        byte  _CX + ((_CH * Z03) >> 10), _CY + ((_CH * Z12) >> 10)
        byte  _CX + ((_CH * Z02) >> 10), _CY + ((_CH * Z13) >> 10)
        byte  _CX + ((_CH * Z01) >> 10), _CY + ((_CH * Z14) >> 10)
    
        byte  _CX - ((_CH * Z00) >> 10), _CY + ((_CH * Z15) >> 10)
        byte  _CX - ((_CH * Z01) >> 10), _CY + ((_CH * Z14) >> 10)
        byte  _CX - ((_CH * Z02) >> 10), _CY + ((_CH * Z13) >> 10)
        byte  _CX - ((_CH * Z03) >> 10), _CY + ((_CH * Z12) >> 10)
        byte  _CX - ((_CH * Z04) >> 10), _CY + ((_CH * Z11) >> 10)
        byte  _CX - ((_CH * Z05) >> 10), _CY + ((_CH * Z10) >> 10)
        byte  _CX - ((_CH * Z06) >> 10), _CY + ((_CH * Z09) >> 10)
        byte  _CX - ((_CH * Z07) >> 10), _CY + ((_CH * Z08) >> 10)
        byte  _CX - ((_CH * Z08) >> 10), _CY + ((_CH * Z07) >> 10)
        byte  _CX - ((_CH * Z09) >> 10), _CY + ((_CH * Z06) >> 10)
        byte  _CX - ((_CH * Z10) >> 10), _CY + ((_CH * Z05) >> 10)
        byte  _CX - ((_CH * Z11) >> 10), _CY + ((_CH * Z04) >> 10)
        byte  _CX - ((_CH * Z12) >> 10), _CY + ((_CH * Z03) >> 10)
        byte  _CX - ((_CH * Z13) >> 10), _CY + ((_CH * Z02) >> 10)
        byte  _CX - ((_CH * Z14) >> 10), _CY + ((_CH * Z01) >> 10)
                                                                  
        byte  _CX - ((_CH * Z15) >> 10), _CY - ((_CH * Z00) >> 10)
        byte  _CX - ((_CH * Z14) >> 10), _CY - ((_CH * Z01) >> 10)
        byte  _CX - ((_CH * Z13) >> 10), _CY - ((_CH * Z02) >> 10)
        byte  _CX - ((_CH * Z12) >> 10), _CY - ((_CH * Z03) >> 10)
        byte  _CX - ((_CH * Z11) >> 10), _CY - ((_CH * Z04) >> 10)
        byte  _CX - ((_CH * Z10) >> 10), _CY - ((_CH * Z05) >> 10)
        byte  _CX - ((_CH * Z09) >> 10), _CY - ((_CH * Z06) >> 10)
        byte  _CX - ((_CH * Z08) >> 10), _CY - ((_CH * Z07) >> 10)
        byte  _CX - ((_CH * Z07) >> 10), _CY - ((_CH * Z08) >> 10)
        byte  _CX - ((_CH * Z06) >> 10), _CY - ((_CH * Z09) >> 10)
        byte  _CX - ((_CH * Z05) >> 10), _CY - ((_CH * Z10) >> 10)
        byte  _CX - ((_CH * Z04) >> 10), _CY - ((_CH * Z11) >> 10)
        byte  _CX - ((_CH * Z03) >> 10), _CY - ((_CH * Z12) >> 10)
        byte  _CX - ((_CH * Z02) >> 10), _CY - ((_CH * Z13) >> 10)
        byte  _CX - ((_CH * Z01) >> 10), _CY - ((_CH * Z14) >> 10)




dat                           
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
 