'' +--------------------------------------------------------------------------+
'' | Cluso's ILI9163 1.44in SPI 128*128 V1.1 LCD Driver                V1.00  |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2015-2016  "Cluso99" (Ray Rodrick)                    |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' RR20161015  Split "ILI9163_spi_060.spin" v0.60 into separate files
''                "LCD_ILI9163_SPI_100.spin"  PASM Cog Driver &
''                "Demo_ILI9163_SPI_060.spin" SPIN Demo program
'' +--------------------------------------------------------------------------+
'' RR20161021       add rotation parameter to Start
''                  fix pasm routine "window" - delete extra "add data,#yoffset"
'' RR20161022 v1.00 Release
''
''+-----------------------------------------------------+
''| Notes:                                              |
''+-----------------------------------------------------+
''  1. Supports both the RED and BLACK LCD boards (set by Start parameters)



CON

' LCD Connector J2: Propeller Pxx
  PIN_3V3       = 0 '   '0         ' 3v3 pwr supplied by prop pin
  PIN_GND       = 1 '   '1         ' GND pwr supplied by prop pin
  PIN_CE        = 2 '0  '2         ' Serial clock enable pin
  PIN_RST       = 3 '1  '3         ' Reset pin
  PIN_DC        = 4 '2  '4         ' Data / Command selection pin
  PIN_SDA       = 5 '3  '5         ' Serial Data pin
  PIN_CLK       = 6 '4  '6         ' Serial Clock pin
  PIN_LED       = 7 '5  '7         ' LED backlight enable pin

' ILI9163C registers...
  LCD_NOP               = $00
  LCD_SW_RESET          = $01   
  LCD_SLEEP_ON          = $10
  LCD_SLEEP_OFF         = $11
  LCD_PARTIAL           = $12
  LCD_NORMAL            = $13
  LCD_INVERSION_OFF     = $20
  LCD_INVERSION_ON      = $21
  LCD_GAMMA_SET         = $26
  LCD_DISPLAY_OFF       = $28
  LCD_DISPLAY_ON        = $29
  LCD_COL_ADR           = $2A
  LCD_ROW_ADR           = $2B
  LCD_RAM_WRITE         = $2C
  LCD_COLOR_SPACE       = $2D   '4K/65K/262K
' LCD_RAMRD             = $2E
  LCD_MADCTR            = $36   'Mem Addr Contol (rotate screen)
  LCD_PIXEL_FORMAT      = $3A
  LCD_FRAME_CTL1        = $B1
  LCD_INVERSION_CTL     = $B4
  LCD_POWER_CTL1        = $C0
  LCD_POWER_CTL2        = $C1
  LCD_VCOM_CTL1         = $C5
  LCD_VCOM_OFFSET       = $C7
  LCD_POS_GAMMA         = $E0
  LCD_NEG_GAMMA         = $E1
  LCD_GAMMA_RS          = $F2
' LCD_ALL_ON            = $A4
' LCD_ALL_OFF           = $A5

  width = 128                   ' screen width  (cols)
  height= 128                   ' screen height (rows)

' BGR 5+6+5 16bits/pixel (RGB reversed)
  black = $0000
  white = $FFFF
  blue  = $F800
  green = $07E0
  red   = $001F
  yellow= red+green
  purple= red+blue


VAR
  long  cog                                             ' pasm cog+1


DAT
''+-----------------------------------------------------+
''| MAILBOX PARAMETERS: Between Spin/Pasm               |
''+-----------------------------------------------------+
' mailbox(es) for PASM cog                             
mailbox         long    0                               '  \ .... (32b)       command + optional param(s)
'cmd            byte    0                               '  | cmd    (8b)        command
'spare          byte    0                               '  | spare  (8b)        ---
'param          word    0                               '  / param  (16b/8b)    16/8/0b param
mailbox1        long    0                               '  \ .... (32b)       optional additional param(s)
'               long    0                               '  / ??     (32b)       32/16/8/0b param(s)
mailbox2        long    $00_00_FF_FF                    '  \      (32b)       screen window in pixels...   
'cs             byte    0                               '  | xs     (8b)        column start
'rs             byte    0                               '  | ys     (8b)        row    start
'ce             byte    127                             '  | xe     (8b)        column end
're             byte    127                             '  / ye     (8b)        row    end
mailbox3        long    blue << 16 | black              '  \      (32b)       screen colors rgb(16b)...
'fgcolor        word    blue                            '  | fgcolor(16b)       foreground color
'bgcolor        word    black                           '  / bgcolor(16b)       background color


''+-----------------------------------------------------+
''| SPIN OBJECT USER INTERFACE ROUTINES                 |
''+-----------------------------------------------------+
PUB Start(pcb, rotate)
' Start the PASM (LCD Driver) cog & Initialised the LCD
  lcdInit(pcb, rotate)
  clearScreen
  return(cog)                                           ' returns cog+1 if successful

PUB Stop
' Stop the PASM (LCD Driver) cog
  if cog
    cogstop(cog~ - 1)

''+-----------------------------------------------------+
''| User High Level Calls                               |
''+-----------------------------------------------------+
PUB clearScreen
' Clear Screen
  setWindow(0,0,width-1,height-1)                       ' full screen 128*128
  fillWindow(mailbox3 & $FFFF)                          ' clear=bgcolor

PUB setFGcolor(rgb)
' Set foreground color
  mailbox3 := (rgb << 16) | (mailbox3 & $FFFF)
  
PUB setBGcolor(rgb)
' Set background color
  mailbox3 := (mailbox3 & $FFFF_0000) | (rgb & $FFFF)
  
PUB setColRow(xs, ys)
' Set Screen Column/Row for subsequent DrawChar
  setWindow(xs, ys, xs+7, ys+7)                         ' set char/font screen window

PUB setWindow(xs, ys, xe, ye)
' Set Screen Window (column start, row start, column end, row end)
  mailbox2 := (xs<<24) | (ys<<16) | (xe<<8) | ye        ' xs, ys, xe, ye 
  mailbox := $81_00_0000
  repeat while mailbox <> 0

PUB fillWindow(rgb) | n, xs, ys, xe, ye
' Fill Window/Rectangle (rgb color)

  xs :=  mailbox2 >> 24
  ys := (mailbox2 >> 16) & $FF
  xe := (mailbox2 >> 8)  & $FF
  ye :=  mailbox2        & $FF
  
  n := (xe - xs +1)*(ye - ys +1)                        ' calc no. of pixels
  mailbox1 := (rgb << 16) | n
  mailbox := $82_00_0000
  repeat while mailbox <> 0

PUB drawChar(char) | xs, ys
' Draw (8*8 font) char @ (xs,ys,xs+7,ys+7), then updates (xs,ys,xe,ye) - uses fgcolor & bgcolor setting

  'get current xs, ys and set xe, ye
  xs :=  mailbox2 >> 24
  ys := (mailbox2 >> 16) & $FF
  setWindow(xs, ys, xs+7, ys+7)                         ' set 8*8 font pixel window

  mailbox  := $83_00_0000 | (char & $7F)                ' paints pixels
  repeat while mailbox <> 0

  'update xs, ys, xe, ye to next posn
  xs += 8
  if xs => width
    xs~
    ys += 8
    if ys => height
     ys~
  setWindow(xs, ys, xs+7, ys+7)                         ' update 8*8 font pixel window ready for next char
     

''+-----------------------------------------------------+
''| User Low Level Calls (directly manipulate the LCD)  |
''+-----------------------------------------------------+
PUB lcdWriteData16(val)
' Write 16b data word to LCD
  lcdWriteData(val >> 8)
  lcdWriteData(val & $FF)

PUB lcdWriteData(val)
' Write 8b data byte to LCD
  mailbox := $100 | val                                 ' dc=data=1
  repeat while mailbox <> 0

PUB lcdWriteCmd(val)
' Write 8b commandbyte to LCD
  mailbox := val                                        ' dc=cmd=0
  repeat while mailbox <> 0

''+-----------------------------------------------------+
''| Initialise LCD 1.44" SPI 128*128 V1.1               |
''+-----------------------------------------------------+
PRI lcdInit(pcb,rotate)         
' Start pasm cog and wait till running
  mailbox := $FF00 + pcb                                ' non-zero (pcb offsety black=0, red=32)
  cog := cognew(@entry, @mailbox) +1                    ' start LCD Driver Cog
  repeat while mailbox <> 0                             ' wait until cog running
  waitcnt(clkfreq/10 + cnt)                             ' 100ms

' Reset the LCD (either HW or SW - both work)
  mailbox := $FF_00_0000                                '\ HW reset LCD
  repeat while mailbox <> 0                             '/
' lcdWriteCmd(LCD_SW_RESET)                             '] SW Reset
  waitcnt(clkfreq/2 + cnt)                              '  500ms

' Sleep mode off
  lcdWriteCmd(LCD_SLEEP_OFF)                           
  waitcnt(clkfreq/1000*5 + cnt)                         ' 5ms

' Pixel Format 16bits/pixel
  lcdWriteCmd(LCD_PIXEL_FORMAT)
  lcdWriteData($05)                                     ' 16b/pixel

{{
Gamma modes not required
  lcdWriteCmd(LCD_GAMMA_SET)
  lcdWriteData($04)                                     ' curve 3

  lcdWriteCmd(LCD_GAMMA_RS)
  lcdWriteData($01)                                     ' gamma adj enabled
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms

  lcdWriteCmd(LCD_POS_GAMMA)
  lcdWriteData($3F)                                     ' 15 params
  lcdWriteData($25)                                    
  lcdWriteData($1C)                                    
  lcdWriteData($1E)                                    
  lcdWriteData($20)                                    
  lcdWriteData($12)                                    
  lcdWriteData($2A)                                    
  lcdWriteData($90)                                    
  lcdWriteData($24)                                    
  lcdWriteData($11)                                    
  lcdWriteData($00)                                    
  lcdWriteData($00)                                    
  lcdWriteData($00)                                    
  lcdWriteData($00)                                    
  lcdWriteData($00)
                                  
  lcdWriteCmd(LCD_NEG_GAMMA)
  lcdWriteData($20)                                     ' 15 params
  lcdWriteData($20)                                    
  lcdWriteData($20)                                    
  lcdWriteData($20)                                    
  lcdWriteData($05)                                    
  lcdWriteData($00)                                    
  lcdWriteData($15)                                    
  lcdWriteData($A7)                                    
  lcdWriteData($3D)                                    
  lcdWriteData($18)                                    
  lcdWriteData($25)                                    
  lcdWriteData($2A)                                    
  lcdWriteData($2B)                                    
  lcdWriteData($2B)                                    
  lcdWriteData($3A)
}}

{{ 
Frame and Power modes default correctly              
  lcdWriteCmd(LCD_FRAME_CTL1)                           ' normal/full
  lcdWriteData($08)                                     ' DIVA=8
  lcdWriteData($08)                                     ' VPA=8
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms
  
  lcdWriteCmd(LCD_INVERSION_CTL)
  lcdWriteData($07)                                     ' NLA=NLB=NLC=1
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms

  lcdWriteCmd(LCD_POWER_CTL1)
  lcdWriteData($0A)                                     ' VRH=10 GVDD=4.30
  lcdWriteData($02)                                     ' VC=2 VCI1=2.65
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms

  lcdWriteCmd(LCD_POWER_CTL2)
  lcdWriteData($02)                                     ' BT=2 AVDD=2xVCI1 VCL=-1xVCI1 VGH=5xVCI1 VGL=-2xVCI1
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms

  lcdWriteCmd(LCD_VCOM_CTL1)
  lcdWriteData($50)                                     ' VMH=80 VCOMH=4.5
  lcdWriteData($5B)                                     ' VML=91 VCOML=-0.225
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms

  lcdWriteCmd(LCD_VCOM_OFFSET)
  lcdWriteData($40)                                     ' NVM=0 VMF=64 VCOMH=VMH VCOML=VML
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms
}}

{{
Row & Cols default correctly 128*128
  lcdWriteCmd(LCD_COL_ADR)
  lcdWriteData($00)                                     ' XSH
  lcdWriteData($00)                                     ' XSL
  lcdWriteData($00)                                     ' XEH
  lcdWriteData($7F)                                     ' XEL (128 pixels wide)

  lcdWriteCmd(LCD_ROW_ADR)
  lcdWriteData($00)                                     ' YSH
  lcdWriteData($00)                                     ' YSL
  lcdWriteData($00)                                     ' YEH
  lcdWriteData($7F)                                     ' YEL (128 pixels high)
}}

' set display rotation
  lcdWriteCmd(LCD_MADCTR)                               ' mem addr ctrl
  lcdWriteData(rotate)                                  ' rotation 0/90/180/270deg=$00/$A0/$C0/$60

' Display On
  lcdWriteCmd(LCD_DISPLAY_ON)
  waitcnt(clkfreq/1000 + cnt)                           ' 1ms


DAT
''+-----------------------------------------------------+
''| PASM LCD Low Level Drivers (separate cog)           |
''+-----------------------------------------------------+
                org     0
entry           mov     hubptr, par                     ' save hub ptr
                mov     hubptr1, hubptr
                add     hubptr1, #4
                mov     hubptr2, hubptr1
                add     hubptr2, #4
                mov     hubptr3, hubptr2
                add     hubptr3, #4
                
                rdlong  offsety, hubptr                 '\ offsety=0(black pcb), offsety=32(red pcb)
                and     offsety, #$FF                   '/

setup           mov     outa, outmask                   ' preset levels
                mov     dira, dirmask                   ' enable outputs
                call    #wait50
                call    #wait50

done            wrlong  zero, hubptr                    ' clear hub mailbox
wait            rdlong  data, hubptr      wz            ' wait for something to do
        if_z    jmp     #wait

                rdlong  data1,  hubptr1                 ' get params(1)
                rdlong  coords, hubptr2                 ' get xs,ys,xe,ye
                rdlong  colors, hubptr3                 ' get fgcolor,bgcolor

                mov     command, data                   '\ extract command
                shr     command, #24                    '|
                cmp     command, #$00     wz            '| $00= write cmd/data from spin
        if_e    jmp     #writex                         '|
                cmp     command, #$FF     wz            '| $FF= HW Reset
        if_e    jmp     #reset                          '|
                cmp     command, #$81     wz            '| $81= Set window xs,ys,xe,ye
        if_e    jmp     #window                         '|
                cmp     command, #$82     wz            '| $82= Fill window rgb
        if_e    jmp     #fill                           '|
                cmp     command, #$83     wz            '| $83= paint char
        if_e    jmp     #paintchar                      '|
                jmp     #done                           '/ unknown command, ignore

                
''+-----------------------------------------------------+
''| LCD SetWindow(xs,ys,xe,ye)                          |
''+-----------------------------------------------------+
window          mov     data, #LCD_COL_ADR
                call    #writecmd
                mov     data, coords                    '\ xs
                shr     data, #24                       '|
               'and     data, #$FF                      '/                      (not reqd)
                call    #write16data

                mov     data, coords                    '\ xe
                shr     data, #8                        '|
                and     data, #$FF                      '/
                call    #write16data

                mov     data, #LCD_ROW_ADR
                call    #writecmd
                mov     data, coords                    '\ ys
                shr     data, #16                       '|
                add     data, offsety                   '| ys+offset
                and     data, #$FF                      '/
                call    #write16data
                
                mov     data, coords                    '\ ye
                add     data, offsety                   '| ye+offset
                and     data, #$FF                      '/
                call    #write16data

                jmp     #done 

''+-----------------------------------------------------+
''| LCD FillWindow(rgb,pixels)                          |
''+-----------------------------------------------------+
fill            mov     data, #LCD_RAM_WRITE
                call    #writecmd
                mov     ctr, data1                      '\ pixels(16b)
                and     ctr, xFFFF                      '/ 
                shr     data1, #16                      ' rgb(16b)
:fill_loop      mov     data, data1                     ' rgb
                call    #write16data
                djnz    ctr, #:fill_loop
                jmp     #done

''+-----------------------------------------------------+
''| LCD PaintChar(char)   mailbox3=fgcolor/bgcolor      |
''+-----------------------------------------------------+
paintchar       mov     fg, colors                      ' get fgcolor/bgcolor
                mov     bg, fg
                shr     fg, #16                         ' extract fgcolor
                and     bg, xFFFF                       ' extract bgcolor
                and     data, #$7F                      ' char =<$7F
                movs    paint4rows, #font               ' point to start of font
                add     paint4rows, data                ' point to first 4 rows of font pixels

                mov     data, #LCD_RAM_WRITE
                call    #writecmd
                
                call    #paint4rows
                add     paint4rows, #128                ' point to second 4 rows of font pixels
                call    #paint4rows
                jmp     #done

paint4rows      mov     pixels, 0-0                     ' get first/second 4 rows of font pixels 
                rev     pixels, #32                     ' reverse font pixels
                mov     ctr, #32                        ' 32 pixels
:paintpixel     rol     pixels, #1              wc
        if_c    mov     data, fg                        ' "1" = fgcolor
        if_nc   mov     data, bg                        ' "0" = bgcolor
                call    #write16data                    ' write fgcolor/bgcolor(16b) pixel(1)
                djnz    ctr, #:paintpixel
paint4rows_ret  ret


''+-----------------------------------------------------+
''| LCD Reset                                           |
''+-----------------------------------------------------+
reset           andn    outa, rstmask                   ' RST=0
                call    #wait50
                or      outa, rstmask                   ' RST=1
                call    #wait50
                jmp     #done

wait50          mov     ctr, cnt
                add     ctr, delay50ms
wait50_ret      ret


''+-----------------------------------------------------+
''| LCD Write 8bit CMD/DATA (b8:cmd=0/data=1)           |
''+-----------------------------------------------------+
writedata       or      data, #$100                     ' DC=1=data
writecmd
write           rol     data, #23                       ' prep 9b byte to write (msb first)
                rol     data, #1                wc      ' cmd/data ?
                muxc    outa, dcmask                    ' cmd=0 / data=1
                andn    outa, csmask                    ' CS=0
                mov     bits, #8
:nextbit        andn    outa, clkmask                   ' CLK=0
                rol     data, #1                wc      ' \ write data bit
                muxc    outa, sdamask                   ' /  SDA=0/1
                or      outa, clkmask                   ' CLK=1
                djnz    bits, #:nextbit
writedata_ret
writecmd_ret
write_ret       ret

writex          call    #write                          ' write cmd/data (from spin)
                jmp     #done

write16data     ror     data, #8                        ' upper8 in b7-0, lower8 in b31-24
                call    #writedata                      ' write upper 8 bits DC=1=data
                rol     data, #8                        ' lower8 in b7-0
                call    #writedata
write16data_ret ret


' font follows in here before "xxxx res 1" (reserved variables)

'' +--------------------------------------------------------------------------+
'' | Font section: 128 characters of 8x8 font                                 |
'' |                two blocks of 128 longs of 4x8 (4 rows of 8 pixels)       |
'' +--------------------------------------------------------------------------+
'' | Derived from: AiChip_SmallFont_Atari_lsb_001.spin                        |
'' |                LSB first version of Font_ATARI from AiGeneric Text Driver|
'' +--------------------------------------------------------------------------+

font          byte      %00000000               ' ........    $00             
              byte      %00000000               ' ........                     
              byte      %00000000               ' ........                     
              byte      %00011000               ' ...##...                     
                                                 
              byte      %00001111               ' ####....    $01
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
                                                 
              byte      %00000000               ' ........    $02  USED FOR CURSOR
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %11111111               ' ########    $03
              byte      %11111111               ' ########
              byte      %11111111               ' ########
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $04
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00001111               ' ####....    $05
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
                                                 
              byte      %11110000               ' ....####    $06
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
                                                 
              byte      %11111111               ' ########    $07
              byte      %11111111               ' ########
              byte      %11111111               ' ########
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $08
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00001111               ' ####....    $09
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
                                                 
              byte      %11110000               ' ....####    $0A
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
                                                 
              byte      %11111111               ' ########    $0B
              byte      %11111111               ' ########
              byte      %11111111               ' ########
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $0C
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00001111               ' ####....    $0D
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
              byte      %00001111               ' ####....
                                                 
              byte      %11110000               ' ....####    $0E
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
              byte      %11110000               ' ....####
                                                 
              byte      %11111111               ' ########    $0F
              byte      %11111111               ' ########
              byte      %11111111               ' ########
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $10
              byte      %10000000               ' .......#
              byte      %10000000               ' .......#
              byte      %10000000               ' .......#
                                                 
              byte      %00000000               ' ........    $11
              byte      %11111111               ' ########
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $12
              byte      %11111111               ' ########
              byte      %10000000               ' .......#
              byte      %10000000               ' .......#
                                                 
              byte      %00000000               ' ........    $13
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $14
              byte      %11111111               ' ########
              byte      %11111111               ' ########
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $15
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %11111000               ' ...#####
                                                 
              byte      %00000000               ' ........    $16
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $17
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00011111               ' #####...
                                                 
              byte      %00011000               ' ...##...    $18
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %11111000               ' ...#####
                                                 
              byte      %00110000               ' ....##..    $19
              byte      %00110000               ' ....##..
              byte      %00110000               ' ....##..
              byte      %00111111               ' ######..
                                                 
              byte      %00100000               ' .....#..    $1A
              byte      %00110000               ' ....##..
              byte      %00111000               ' ...###..
              byte      %00111100               ' ..####..
                                                 
              byte      %00000100               ' ..#.....    $1B
              byte      %00001100               ' ..##....
              byte      %00011100               ' ..###...
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $1C
              byte      %00011000               ' ...##...
              byte      %00111100               ' ..####..
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $1D
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $1E
              byte      %00001000               ' ...#....
              byte      %00001100               ' ..##....
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $1F
              byte      %00010000               ' ....#...
              byte      %00110000               ' ....##..
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $20  Space
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $21  !
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $22  "
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $23  #
              byte      %01100110               ' .##..##.
              byte      %11111111               ' ########
              byte      %01100110               ' .##..##.
                                                 
              byte      %00011000               ' ...##...    $24  $
              byte      %01111100               ' ..#####.
              byte      %00000110               ' .##.....
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $25  %
              byte      %01100110               ' .##..##.
              byte      %00110110               ' .##.##..
              byte      %00011000               ' ...##...
                                                 
              byte      %00111000               ' ...###..    $26  &
              byte      %01101100               ' ..##.##.
              byte      %00111000               ' ...###..
              byte      %00011100               ' ..###...
                                                 
              byte      %00000000               ' ........    $27  '
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $28  (
              byte      %01110000               ' ....###.
              byte      %00111000               ' ...###..
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $29  )
              byte      %00001110               ' .###....
              byte      %00011100               ' ..###...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $2A  *
              byte      %01100110               ' .##..##.
              byte      %00111100               ' ..####..
              byte      %11111111               ' ########
                                                 
              byte      %00000000               ' ........    $2B  +
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $2C  ,
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $2D  -
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $2E  .
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $2F  /
              byte      %01100000               ' .....##.
              byte      %00110000               ' ....##..
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $30  0
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %01110110               ' .##.###.
                                                 
              byte      %00000000               ' ........    $31  1
              byte      %00011000               ' ...##...
              byte      %00011100               ' ..###...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $32  2
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $33  3
              byte      %01111110               ' .######.
              byte      %00110000               ' ....##..
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $34  4
              byte      %00110000               ' ....##..
              byte      %00111000               ' ...###..
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $35  5
              byte      %01111110               ' .######.
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $36  6
              byte      %00111100               ' ..####..
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $37  7
              byte      %01111110               ' .######.
              byte      %01100000               ' .....##.
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $38  8
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $39  9
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %01111100               ' ..#####.
                                                 
              byte      %00000000               ' ........    $3A  :
              byte      %00000000               ' ........
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $3B  ;
              byte      %00000000               ' ........
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %01100000               ' .....##.    $3C  <
              byte      %00110000               ' ....##..
              byte      %00011000               ' ...##...
              byte      %00001100               ' ..##....
                                                 
              byte      %00000000               ' ........    $3D  =
              byte      %00000000               ' ........
              byte      %01111110               ' .######.
              byte      %00000000               ' ........
                                                 
              byte      %00000110               ' .##.....    $3E  >
              byte      %00001100               ' ..##....
              byte      %00011000               ' ...##...
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $3F  ?
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $40  @
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %01110110               ' .##.###.
                                                 
              byte      %00000000               ' ........    $41  A
              byte      %00011000               ' ...##...
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $42  B
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $43  C
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %00000110               ' .##.....
                                                 
              byte      %00000000               ' ........    $44  D
              byte      %00011110               ' .####...
              byte      %00110110               ' .##.##..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $45  E
              byte      %01111110               ' .######.
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $46  F
              byte      %01111110               ' .######.
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $47  G
              byte      %01111100               ' ..#####.
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
                                                 
              byte      %00000000               ' ........    $48  H
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $49  I
              byte      %01111110               ' .######.
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $4A  J
              byte      %01100000               ' .....##.
              byte      %01100000               ' .....##.
              byte      %01100000               ' .....##.
                                                 
              byte      %00000000               ' ........    $4B  K
              byte      %01100110               ' .##..##.
              byte      %00110110               ' .##.##..
              byte      %00011110               ' .####...
                                                 
              byte      %00000000               ' ........    $4C  L
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
                                                 
              byte      %00000000               ' ........    $4D  M
              byte      %11000110               ' .##...##
              byte      %11101110               ' .###.###
              byte      %11111110               ' .#######
                                                 
              byte      %00000000               ' ........    $4E  N
              byte      %01100110               ' .##..##.
              byte      %01101110               ' .###.##.
              byte      %01111110               ' .######.
                                                 
              byte      %00000000               ' ........    $4F  O
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $50  P
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $51  Q
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $52  R
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $53  S
              byte      %00111100               ' ..####..
              byte      %00000110               ' .##.....
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $54  T
              byte      %01111110               ' .######.
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $55  U
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $56  V
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $57  W
              byte      %11000110               ' .##...##
              byte      %11000110               ' .##...##
              byte      %11010110               ' .##.#.##
                                                 
              byte      %00000000               ' ........    $58  X
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $59  Y
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $5A  Z
              byte      %01111110               ' .######.
              byte      %00110000               ' ....##..
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $5B  [
              byte      %01111000               ' ...####.
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $5C  \
              byte      %00000010               ' .#......
              byte      %00000110               ' .##.....
              byte      %00001100               ' ..##....
                                                 
              byte      %00000000               ' ........    $5D  ]
              byte      %00011110               ' .####...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $5E  ^
              byte      %00010000               ' ....#...
              byte      %00111000               ' ...###..
              byte      %01101100               ' ..##.##.
                                                 
              byte      %00000000               ' ........    $5F  _
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $60  `
              byte      %10000000               ' .......#
              byte      %11000000               ' ......##
              byte      %01100000               ' .....##.
                                                 
              byte      %00000000               ' ........    $61  a
              byte      %00000000               ' ........
              byte      %00111100               ' ..####..
              byte      %01100000               ' .....##.
                                                 
              byte      %00000000               ' ........    $62  b
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $63  c
              byte      %00000000               ' ........
              byte      %00111100               ' ..####..
              byte      %00000110               ' .##.....
                                                 
              byte      %00000000               ' ........    $64  d
              byte      %01100000               ' .....##.
              byte      %01100000               ' .....##.
              byte      %01111100               ' ..#####.
                                                 
              byte      %00000000               ' ........    $65  e
              byte      %00000000               ' ........
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $66  f
              byte      %01110000               ' ....###.
              byte      %00011000               ' ...##...
              byte      %01111100               ' ..#####.
                                                 
              byte      %00000000               ' ........    $67  g
              byte      %00000000               ' ........
              byte      %01111100               ' ..#####.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $68  h
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
              byte      %00111110               ' .#####..
                                                 
              byte      %00000000               ' ........    $69  i
              byte      %00011000               ' ...##...
              byte      %00000000               ' ........
              byte      %00011100               ' ..###...
                                                 
              byte      %00000000               ' ........    $6A  j
              byte      %00110000               ' ....##..
              byte      %00000000               ' ........
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $6B  k
              byte      %00000110               ' .##.....
              byte      %00000110               ' .##.....
              byte      %00110110               ' .##.##..
                                                 
              byte      %00000000               ' ........    $6C  l
              byte      %00011100               ' ..###...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $6D  m
              byte      %00000000               ' ........
              byte      %01100110               ' .##..##.
              byte      %11111110               ' .#######
                                                 
              byte      %00000000               ' ........    $6E  n
              byte      %00000000               ' ........
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $6F  o
              byte      %00000000               ' ........
              byte      %00111100               ' ..####..
              byte      %01100110               ' .##..##.
                                                  
              byte      %00000000               ' ........    $70  p
              byte      %00000000               ' ........
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $71  q
              byte      %00000000               ' ........
              byte      %01111100               ' ..#####.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $72  r
              byte      %00000000               ' ........
              byte      %00111110               ' .#####..
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $73  s
              byte      %00000000               ' ........
              byte      %01111100               ' ..#####.
              byte      %00000110               ' .##.....
                                                 
              byte      %00000000               ' ........    $74  t
              byte      %00011000               ' ...##...
              byte      %01111110               ' .######.
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $75  u
              byte      %00000000               ' ........
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                
              byte      %00000000               ' ........    $76  v
              byte      %00000000               ' ........
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $77  w
              byte      %00000000               ' ........
              byte      %11000110               ' .##...##
              byte      %11010110               ' .##.#.##
                                                 
              byte      %00000000               ' ........    $78  x
              byte      %00000000               ' ........
              byte      %01100110               ' .##..##.
              byte      %00111100               ' ..####..
                                                 
              byte      %00000000               ' ........    $79  y
              byte      %00000000               ' ........
              byte      %01100110               ' .##..##.
              byte      %01100110               ' .##..##.
                                                 
              byte      %00000000               ' ........    $7A  z
              byte      %00000000               ' ........
              byte      %01111110               ' .######.
              byte      %00110000               ' ....##..
                                                 
              byte      %00000000               ' ........    $7B  {
              byte      %00111000               ' ...###..
              byte      %00011000               ' ...##...
              byte      %00011110               ' .###....
                                                 
              byte      %00011000               ' ...##...    $7C  |
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
              byte      %00011000               ' ...##...
                                                 
              byte      %00000000               ' ........    $7D  }
              byte      %00011100               ' ..###...
              byte      %00011000               ' ...##...
              byte      %01111000               ' ....###.
                                                 
              byte      %00000000               ' ........    $7E  ~
              byte      %11001100               ' ..##..##
              byte      %01111110               ' .######.
              byte      %00110011               ' ##..##..
                                                 
              byte      %00000000               ' ........    $7F
              byte      %00011100               ' ..###...
              byte      %00100010               ' .#...#..
              byte      %00100010               ' .#...#..

' *******************************************************************************************************
' second half of font longs...

              byte      %00011000               ' ...##...    $00
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $01
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........
                                                 
              byte      %00000000               ' ........    $02  USED FOR CURSOR
              byte      %00000000               ' ........
              byte      %11111110               ' .#######
              byte      %11111110               ' .####### 
                                                 
              byte      %00000000               ' ........    $03                          
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00001111               ' ####....    $04                          
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
              byte      %00001111               '.####....                                 
                                                                                           
              byte      %00001111               ' ####....    $05                          
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
                                                                                           
              byte      %00001111               ' ####....    $06                          
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
                                                                                           
              byte      %00001111               ' ####....    $07                          
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
              byte      %00001111               ' ####....                                 
                                                                                           
              byte      %11110000               ' ....####    $08                          
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
                                                                                           
              byte      %11110000               ' ....####    $09                          
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
                                                                                           
              byte      %11110000               ' ....####    $0A                          
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
                                                                                           
              byte      %11110000               ' ....####    $0B                          
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
              byte      %11110000               ' ....####                                 
                                                                                           
              byte      %11111111               ' ########    $0C                          
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
              byte      %11111111               '.########                                 
                                                                                           
              byte      %11111111               ' ########    $0D                          
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
                                                                                           
              byte      %11111111               ' ########    $0E                          
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
                                                                                           
              byte      %11111111               ' ########    $0F                          
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
                                                                                           
              byte      %10000000               ' .......#    $10                          
              byte      %10000000               ' .......#                                 
              byte      %11111111               ' ########                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $11                          
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %10000000               ' .......#    $12                          
              byte      %10000000               ' .......#                                 
              byte      %10000000               ' .......#                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $13                          
              byte      %00000000               ' ........                                 
              byte      %11111111               ' ########                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11111111               ' ########    $14                          
              byte      %11111111               ' ########                                 
              byte      %11111111               ' ########                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11111000               ' ...#####    $15                          
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
                                                                                           
              byte      %11111111               ' ########    $16                          
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011111               ' #####...    $17                          
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
                                                                                           
              byte      %11111000               ' ...#####    $18                          
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111111               ' ######..    $19                          
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111000               ' ...###..    $1A                          
              byte      %00110000               ' ....##..                                 
              byte      %00100000               ' .....#..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011100               ' ..###...    $1B                          
              byte      %00001100               ' ..##....                                 
              byte      %00000100               ' ..#.....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $1C                          
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111110               ' .######.    $1D                          
              byte      %00111100               ' ..####..                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111110               ' .######.    $1E                          
              byte      %00001100               ' ..##....                                 
              byte      %00001000               ' ...#....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111110               ' .######.    $1F                          
              byte      %00110000               ' ....##..                                 
              byte      %00010000               ' ....#...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $20  Space                   
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $21  !                       
              byte      %00000000               ' ........                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $22  "                       
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $23  #                       
              byte      %11111111               ' ########                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100000               ' .....##.    $24  $                       
              byte      %00111110               ' .#####..                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00001100               ' ..##....    $25  %                       
              byte      %01100110               ' .##..##.                                 
              byte      %01100010               ' .#...##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11110110               ' .##.####    $26  &                       
              byte      %01100110               ' .##..##.                                 
              byte      %11011100               ' ..###.##                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $27  '                  
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $28  (                       
              byte      %00111000               ' ...###..                                 
              byte      %01110000               ' ....###.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $29  )                       
              byte      %00011100               ' ..###...                                 
              byte      %00001110               ' .###....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111100               ' ..####..    $2A  *                       
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $2B  +                       
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $2C  ,                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00001100               ' ..##....                                 
                                                                                           
              byte      %00000000               ' ........    $2D  -                       
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $2E  .                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00001100               ' ..##....    $2F  /                       
              byte      %00000110               ' .##.....                                 
              byte      %00000010               ' .#......                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01101110               ' .###.##.    $30  0                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $31  1                       
              byte      %00011000               ' ...##...                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $32  2                       
              byte      %00001100               ' ..##....                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00110000               ' ....##..    $33  3                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00110110               ' .##.##..    $34  4                       
              byte      %01111110               ' .######.                                 
              byte      %00110000               ' ....##..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100000               ' .....##.    $35  5                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $36  6                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $37  7                       
              byte      %00001100               ' ..##....                                 
              byte      %00001100               ' ..##....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $38  8                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100000               ' .....##.    $39  9                       
              byte      %00110000               ' ....##..                                 
              byte      %00011100               ' ..###...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $3A  :                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $3B  ;                       
              byte      %00011000               ' ...##...                                 
              byte      %00001100               ' ..##....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $3C  <                       
              byte      %00110000               ' ....##..                                 
              byte      %01100000               ' .....##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $3D  =                       
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $3E  >                       
              byte      %00001100               ' ..##....                                 
              byte      %00000110               ' .##.....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $3F  ?                       
              byte      %00000000               ' ........                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01110110               ' .##.###.    $40  @                       
              byte      %00000110               ' .##.....                                 
              byte      %01111100               ' ..#####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $41  A                       
              byte      %01111110               ' .######.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $42  B                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111110               ' .#####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000110               ' .##.....    $43  C                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $44  D                       
              byte      %00110110               ' .##.##..                                 
              byte      %00011110               ' .####...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000110               ' .##.....    $45  E                       
              byte      %00000110               ' .##.....                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000110               ' .##.....    $46  F                       
              byte      %00000110               ' .##.....                                 
              byte      %00000110               ' .##.....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01110110               ' .##.###.    $47  G                       
              byte      %01100110               ' .##..##.                                 
              byte      %01111100               ' ..#####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $48  H                       
              byte      %01100110               ' .##..##.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $49  I                       
              byte      %00011000               ' ...##...                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100000               ' .....##.    $4A  J                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011110               ' .####...    $4B  K                       
              byte      %00110110               ' .##.##..                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000110               ' .##.....    $4C  L                       
              byte      %00000110               ' .##.....                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11010110               ' .##.#.##    $4D  M                       
              byte      %11000110               ' .##...##                                 
              byte      %11000110               ' .##...##                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111110               ' .######.    $4E  N                       
              byte      %01110110               ' .##.###.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $4F  O                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111110               ' .#####..    $50  P                       
              byte      %00000110               ' .##.....                                 
              byte      %00000110               ' .##.....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $51  Q                       
              byte      %00110110               ' .##.##..                                 
              byte      %01101100               ' ..##.##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111110               ' .#####..    $52  R                       
              byte      %00110110               ' .##.##..                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100000               ' .....##.    $53  S                       
              byte      %01100000               ' .....##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $54  T                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $55  U                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $56  V                       
              byte      %00111100               ' ..####..                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11111110               ' .#######    $57  W                       
              byte      %11101110               ' .###.###                                 
              byte      %11000110               ' .##...##                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111100               ' ..####..    $58  X                       
              byte      %01100110               ' .##..##.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $59  Y                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00001100               ' ..##....    $5A  Z                       
              byte      %00000110               ' .##.....                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $5B  [                       
              byte      %00011000               ' ...##...                                 
              byte      %01111000               ' ...####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $5C  \                       
              byte      %00110000               ' ....##..                                 
              byte      %01100000               ' .....##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $5D  ]                       
              byte      %00011000               ' ...##...                                 
              byte      %00011110               ' .####...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11000110               ' .##...##    $5E  ^                       
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $5F  _                       
              byte      %00000000               ' ........                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00110011               ' ##..##..    $60  `                       
              byte      %00011110               ' .####...                                 
              byte      %00001100               ' ..##....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111100               ' ..#####.    $61  a                       
              byte      %01100110               ' .##..##.                                 
              byte      %01111100               ' ..#####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $62  b                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111110               ' .#####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000110               ' .##.....    $63  c                       
              byte      %00000110               ' .##.....                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $64  d                       
              byte      %01100110               ' .##..##.                                 
              byte      %01111100               ' ..#####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01111110               ' .######.    $65  e                       
              byte      %00000110               ' .##.....                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $66  f                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $67  g                       
              byte      %01111100               ' ..#####.                                 
              byte      %01100000               ' .....##.                                 
              byte      %00111110               ' .#####..                                 
                                                                                           
              byte      %01100110               ' .##..##.    $68  h                       
              byte      %01100110               ' .##..##.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $69  i                       
              byte      %00011000               ' ...##...                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00110000               ' ....##..    $6A  j                       
              byte      %00110000               ' ....##..                                 
              byte      %00110000               ' ....##..                                 
              byte      %00011110               ' .####...                                 
                                                                                           
              byte      %00011110               ' .####...    $6B  k                       
              byte      %00110110               ' .##.##..                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $6C  l                       
              byte      %00011000               ' ...##...                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11111110               ' .#######    $6D  m                       
              byte      %11010110               ' .##.#.##                                 
              byte      %11000110               ' .##...##                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $6E  n                       
              byte      %01100110               ' .##..##.                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $6F  o                       
              byte      %01100110               ' .##..##.                                 
              byte      %00111100               ' ..####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $70  p                       
              byte      %00111110               ' .#####..                                 
              byte      %00000110               ' .##.....                                 
              byte      %00000110               ' .##.....                                 
                                                                                           
              byte      %01100110               ' .##..##.    $71  q                       
              byte      %01111100               ' ..#####.                                 
              byte      %01100000               ' .....##.                                 
              byte      %01100000               ' .....##.                                 
                                                                                           
              byte      %00000110               ' .##.....    $72  r                       
              byte      %00000110               ' .##.....                                 
              byte      %00000110               ' .##.....                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00111100               ' ..####..    $73  s                       
              byte      %01100000               ' .....##.                                 
              byte      %00111110               ' .#####..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $74  t                       
              byte      %00011000               ' ...##...                                 
              byte      %01110000               ' ....###.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $75  u                       
              byte      %01100110               ' .##..##.                                 
              byte      %01111100               ' ..#####.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $76  v                       
              byte      %00111100               ' ..####..                                 
              byte      %00011000               ' ...##...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %11111110               ' .#######    $77  w                       
              byte      %01111100               ' ..#####.                                 
              byte      %01101100               ' ..##.##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $78  x                       
              byte      %00111100               ' ..####..                                 
              byte      %01100110               ' .##..##.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %01100110               ' .##..##.    $79  y                       
              byte      %01111100               ' ..#####.                                 
              byte      %00110000               ' ....##..                                 
              byte      %00011110               ' .####...                                 
                                                                                           
              byte      %00011000               ' ...##...    $7A  z                       
              byte      %00001100               ' ..##....                                 
              byte      %01111110               ' .######.                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011110               ' .###....    $7B  {                       
              byte      %00011000               ' ...##...                                 
              byte      %00111000               ' ...###..                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011000               ' ...##...    $7C  |                       
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
              byte      %00011000               ' ...##...                                 
                                                                                           
              byte      %01111000               ' ....###.    $7D  }                       
              byte      %00011000               ' ...##...                                 
              byte      %00011100               ' ..###...                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00000000               ' ........    $7E  ~                       
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
              byte      %00000000               ' ........                                 
                                                                                           
              byte      %00011100               ' ..###...    $7F                          
              byte      %00000000               ' ........
              byte      %00000000               ' ........
              byte      %00000000               ' ........

' end of font
' *******************************************************************************************************

''+-----------------------------------------------------+
''| PASM Constants & Variables                          |
''+-----------------------------------------------------+

outmask         long    1<<PIN_3V3 | 0<<PIN_GND | 1<<PIN_CE | 1<<PIN_RST | 0<<PIN_DC | 0<<PIN_SDA | 1<<PIN_CLK | 1<<PIN_LED
dirmask         long    1<<PIN_3V3 | 1<<PIN_GND | 1<<PIN_CE | 1<<PIN_RST | 1<<PIN_DC | 1<<PIN_SDA | 1<<PIN_CLK | 1<<PIN_LED
csmask          long    1<<PIN_CE
rstmask         long    1<<PIN_RST
dcmask          long    1<<PIN_DC
sdamask         long    1<<PIN_SDA
clkmask         long    1<<PIN_CLK

delay50ms       long    104_000_000 / 1000 * 50         ' 50ms at 96MHz (will be a bit longer if <104MHz)
zero            long    0
xFFFF           long    $FFFF
offsety         long    0                               ' offsety=0(black pcb), offsety=32(red pcb)

data            res     1
data1           res     1
coords          res     1                 ' xs, ys, xe, ye
colors          res     1                 ' fgcolor, bgcolor
fg              res     1
bg              res     1
command         res     1
hubptr          res     1
hubptr1         res     1
hubptr2         res     1
hubptr3         res     1
pixels          res     1
bits            res     1
ctr             res     1

                FIT     496
                